/***********************/
/*
/* P2X 2005 v1.0
/* -------------
/*
/* 
/* Authors
/* -------
/*
/* Karim Benmachiche - karim@0wned.ch
/* Michel Laskowski - doesnt@want.spam
/*
/*
/* Changelog
/* ---------
/*
/* 04.06.05 - P2X 2005 v1.0
/* - Key signatures with [] are outputted correctly in the misc field in the beginning of the XML file.
/* - Created options.
/* - Warning is printed in case a neumatic note is encountered.
/* - All special measure bars are handled correctly.
/* - Using begin instead of start for beamings.
/* - Created an error/warning log file.
/* - All memory problems fixed.
/* - All time signatures work properly, except the ones that have different meanings (2, 3, c3, o)
/* - Fixed a huge bug with alterations, it might need some testing...
/* - Rhytmic models are handled.
/* - No more normal global variables for pointers and id's to prevent fuckups.
/* - Changed name. :)
/*
/* 23.05.05 - rismpeatoxml v0.3
/* - Tried to make it work on the whole db but crashes after a while - memory problems...
/* - Notes and measures repitions are supported. Need testing!
/* - Accidentals alterations are now reported through the whole measure, unless a natural occurs of course.
/* - Added support for clef/time signature/key signature change inside the incipit.
/* - Incipit ID is stored in a var, might be useful.
/* - Measure bars are detected properly, except for incipit nb 400090279.
/* - Measure rests are handled.
/* - Grace notes, slurs, fermatas, trills are supported.
/* - Created functions to add notes and attributes to the data structure.
/* - Tuplets are implemented but need testing.
/* - Divisions and durations are calculated properly, some testing needed though.
/* - Fixed the octaves conversion, works with lower ones as well now.
/* - Having the last measure incomplete is actually not a problem in MusicXML, so nothing to do about it.
/* - Fixed the attributes flag which was opening several attributes fields instead of only one.
/* - Fixed one of the alterations output which was looking for a 'R' instead of a 'A'.
/*
/* 17.05.05 - rismpeatoxml v0.2
/* - Works with x measures now. Although it can't handle yet if the last measure is incomplete.
/* - Key signature is now found through a function.
/* - All warnings on compile have been fixed.
/* - Alterations, accidentals and dots work.
/* - Note rests work, measure rests are still to do.
/* - All the rest is still to do.
/*
/* 11.05.05 - rismpeatoxml v0.1
/* - The basics work. Stops after one measure, time signature doesnt work properly.
/* - No note attribute has been implemented yet (accidental, dots, beamings etc).
/* - Divisions and durations of measures and notes aren't done either.
/* - Rests, rhytmic models,repetions still to do as well.
/* - Input and output still need to be worked on.
/* - Memory isn't freed yet.
/* - Find a cool name :P
/*
/* Compile:
/* gcc p2x2005_10.c -o p2x2005_10.exe
/*
/* Run:
/* ./p2x2005_10.exe -[options]f <input_file>
/***********************/

#include <ctype.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#define bool unsigned int
#define true 1
#define false 0

FILE* errlog;

/******************************************************************/
/*                                                                */
/*                         DATA STRUCTURE                         */
/*                                                                */
/******************************************************************/

struct rhythm_str
{
	int beat;
	int beat_type;
	bool common;
	bool mensural;
};
typedef struct rhythm_str rhythm;

/**/
/* Attributes id's are:
/* 1: key signature
/* 2: measure bar
/* 3: measure rest
/*
/* 10: G clef
/* 11: F clef
/* 12: C clef
/* 13: g clef (octave change)
/* 
/* 20: dotted note
/* 21: accidental
/* 22: fermata
/* 23: trill
/* 24: slashed grace
/* 25: unslashed grace
/* 26: starting slur
/* 27: ending slur
/*
/* 30: opening tuplet
/* 31: continuing tuplet
/* 32: closing tuplet
/*
/* 40: opening beaming
/* 41: continuing beaming
/* 42: closing beaming
/*
/* 50: double measure bar
/* 51: final double measure bar
/* 52: backward repeat sign
/* 53: forward repeat sign
/* 54: final forward repeat sign
/*
/**/

struct attribute_str
{
	int id;
	int value;
	
	struct attribute_str* next;
};
typedef struct attribute_str attribute;

struct attribute_list
{
	attribute *start;
	attribute *end;	
};

struct note_str
{
	char step;
	int octave;
	float rhythm;
	struct attribute_list note_attributes;

	struct note_str* next;
};
typedef struct note_str note;

struct note_list
{
	note *start;
	note *end;	
};

struct measure_str
{
	int number;
	rhythm *time_signature;
	struct note_list notes;
	struct attribute_list measure_attributes;

	struct measure_str* next;
};
typedef struct measure_str measure;


/******************************************************************/
/*                                                                */
/*                        MEASURE CREATION                        */
/*                                                                */
/******************************************************************/

measure* create_measure(int number, measure* next_measure)
{
	measure* new_measure;
	new_measure = (measure*)malloc(sizeof(measure)*10000);
	(*new_measure).number = number;
	(*new_measure).notes.start = NULL;
	(*new_measure).notes.end = NULL;
	(*new_measure).time_signature = NULL;	
	(*new_measure).measure_attributes.start = NULL;
	(*new_measure).measure_attributes.end = NULL;
	(*new_measure).next = next_measure;
	return new_measure;
}

/******************************************************************/
/*                                                                */
/*                         NOTE  CREATION                         */
/*                                                                */
/******************************************************************/

note* create_note(char step, int octave, float rhythm, note* next_note)
{
	note* new_note;
	new_note = (note*)malloc(sizeof(note)*10000);
	(*new_note).step = step;
	(*new_note).octave = octave;	
	(*new_note).rhythm = rhythm;
	(*new_note).note_attributes.start = NULL;
	(*new_note).note_attributes.end = NULL;
	(*new_note).next = next_note;
	return new_note;
}

/******************************************************************/
/*                                                                */
/*                       ATTRIBUTE CREATION                       */
/*                                                                */
/******************************************************************/

attribute* create_attribute(int id, int value, attribute* next_attribute)
{
	attribute* new_attribute;
	new_attribute = (attribute*)malloc(sizeof(attribute)*10000);
	(*new_attribute).id = id;
	(*new_attribute).value = value;	
	(*new_attribute).next = next_attribute;
	
	return new_attribute;
}

/******************************************************************/
/*                                                                */
/*                        RHYTHM  CREATION                        */
/*                                                                */
/******************************************************************/

rhythm* create_rhythm(int beat, int beat_type, bool common, bool mensural)
{
	rhythm* new_rhythm;
	new_rhythm = (rhythm*)malloc(sizeof(rhythm)*10000);
	(*new_rhythm).beat = beat;
	(*new_rhythm).beat_type = beat_type;	
	(*new_rhythm).common = common;
	(*new_rhythm).mensural = mensural;
	return new_rhythm;
}

/******************************************************************/
/*                                                                */
/*                         NOTE  ADDITION                         */
/*                                                                */
/******************************************************************/

void add_note(note* temp_note, measure* measure_update)
{
	if((*measure_update).notes.end == NULL)
	{
		(*measure_update).notes.start = temp_note;
		(*measure_update).notes.end = temp_note;
	}
	else
	{
		((note *)measure_update->notes.end)->next = temp_note;
		measure_update->notes.end = temp_note;
	}
}

/******************************************************************/
/*                                                                */
/*                   MEASURE ATTRIBUTE ADDITION                   */
/*                                                                */
/******************************************************************/

void add_measure_attrib(attribute* temp_attrib, measure* measure_update)
{
	if((*measure_update).measure_attributes.end == NULL)
	{
		(*measure_update).measure_attributes.start = temp_attrib;
		(*measure_update).measure_attributes.end = temp_attrib;
	}
	else
	{
		((attribute*)measure_update->measure_attributes.end)->next = temp_attrib;
		measure_update->measure_attributes.end = temp_attrib;
	}
}

/******************************************************************/
/*                                                                */
/*                    NOTE ATTRIBUTE ADDITION                     */
/*                                                                */
/******************************************************************/

void add_note_attrib(attribute* temp_attrib, note* tmp)
{
	if((*tmp).note_attributes.end == NULL)
	{
		(*tmp).note_attributes.start = temp_attrib;
		(*tmp).note_attributes.end = temp_attrib;
	}
	else
	{
		((attribute*)tmp->note_attributes.end)->next = temp_attrib;
		tmp->note_attributes.end = temp_attrib;
	}
}

/******************************************************************/
/*                                                                */
/*                         ATTRIBUTE COPY                         */
/*                                                                */
/******************************************************************/

attribute* copy_attribute(attribute* old_attribute)
{
	attribute* new_attribute;
	new_attribute = (attribute*)malloc(sizeof(attribute)*10000);
	(*new_attribute).id = (*old_attribute).id;
	(*new_attribute).value = (*old_attribute).value;	
	(*new_attribute).next = NULL;
	return new_attribute;
}

/******************************************************************/
/*                                                                */
/*                           NOTE  COPY                           */
/*                                                                */
/******************************************************************/

note* copy_note(note* old_note)
{
	note* new_note;
	attribute* tmp_attrib;
	new_note = (note*)malloc(sizeof(note)*10000);
	(*new_note).step = (*old_note).step;
	(*new_note).octave = (*old_note).octave;
	(*new_note).rhythm = (*old_note).rhythm;
	(*new_note).note_attributes.start = NULL;
	(*new_note).note_attributes.end = NULL;
	
	tmp_attrib = (*old_note).note_attributes.start;
	if (tmp_attrib != NULL)
	{
		while (tmp_attrib != NULL)
		{
			attribute* new_attrib;
			new_attrib = copy_attribute(tmp_attrib);
			add_note_attrib(new_attrib, new_note);
			tmp_attrib = (*tmp_attrib).next;
			new_attrib = NULL;
		}
	}
	tmp_attrib = NULL;
	
	(*new_note).next = NULL;
	return new_note;
}

/******************************************************************/
/*                                                                */
/*                          MEASURE COPY                          */
/*                                                                */
/******************************************************************/

measure* copy_measure(measure* old_measure)
{
	measure* new_measure;
	note*	tmp_note;
	attribute* tmp_attrib;
	new_measure = (measure*)malloc(sizeof(measure)*10000);
	(*new_measure).number = (*old_measure).number+1;
	(*new_measure).time_signature = NULL;
	(*new_measure).notes.start = NULL;
	(*new_measure).notes.end = NULL;	
	(*new_measure).measure_attributes.start = NULL;
	(*new_measure).measure_attributes.end = NULL;	

	tmp_note = (*old_measure).notes.start;
	if (tmp_note != NULL)
	{
		while (tmp_note != NULL)
		{
			note* new_note;
			new_note = copy_note(tmp_note);
			add_note(new_note, new_measure);
			tmp_note = (*tmp_note).next;
			new_note = NULL;
		}
	}
	tmp_note = NULL;
	
	tmp_attrib = (*old_measure).measure_attributes.start;
	if (tmp_attrib != NULL)
	{
		while (tmp_attrib != NULL)
		{
			attribute* new_attrib;
			new_attrib = copy_attribute(tmp_attrib);
			add_measure_attrib(new_attrib, new_measure);
			tmp_attrib = (*tmp_attrib).next;
			new_attrib = NULL;
		}
	}
	tmp_attrib = NULL;
	
	(*new_measure).next = NULL;
	return new_measure;
}

/******************************************************************/
/*                                                                */
/*                          MEMORY  FREE                          */
/*                                                                */
/******************************************************************/

void destroy_rhythm(rhythm* rip_rhythm)
{
	free(rip_rhythm);
	rip_rhythm = NULL;
}

void destroy_attribute(attribute* rip_attribute)
{
	if ((*rip_attribute).next != NULL) destroy_attribute((*rip_attribute).next);

	free(rip_attribute);
	rip_attribute = NULL;
}

void destroy_note(note* rip_note)
{
	if ((*rip_note).next != NULL) destroy_note((*rip_note).next);
	if ((*rip_note).note_attributes.start != NULL) destroy_attribute((*rip_note).note_attributes.start);
	
	(*rip_note).note_attributes.start = NULL;
	(*rip_note).note_attributes.end = NULL;

	free(rip_note);
	rip_note = NULL;
}

void destroy_measure(measure* rip_measure)
{
	if ((*rip_measure).next != NULL) destroy_measure((*rip_measure).next);
	if ((*rip_measure).notes.start != NULL) destroy_note((*rip_measure).notes.start);
	if ((*rip_measure).measure_attributes.start != NULL) destroy_attribute((*rip_measure).measure_attributes.start);
	if ((*rip_measure).time_signature != NULL) destroy_rhythm((*rip_measure).time_signature);
	
	(*rip_measure).notes.start = NULL;
	(*rip_measure).notes.end = NULL;
	(*rip_measure).measure_attributes.start = NULL;
	(*rip_measure).measure_attributes.end = NULL;

	free(rip_measure);
	rip_measure = NULL;
}

/******************************************************************/
/*                                                                */
/*                         MISC TOOLS                             */
/*                                                                */
/******************************************************************/

rhythm* rhythm_convert(void *time_signature_v)
{
	//tricky trick
	char *time_signature = (char*) time_signature_v;
	int beat, beat_type;
	bool common, mensural;

	//perfect time
	if (time_signature[0] == 'o')
	{
		common = false;
		mensural = true;
		beat = 0;
		beat_type = 0;
	}
	//commons
	else if (time_signature[0] == 'c')
	{
		//c3 c3/2
		if (time_signature[1] == '3')
		{
			mensural = true;
			common = false;
			beat = 0;
			beat_type = 0;
		}
		else
		{
			//c c/
			mensural = false;
			common = true;
			if (time_signature[1] == '/')
			{
				beat = 2;
				beat_type = 2;
			}
			else
			{
				beat = 4;
				beat_type = 4;
			}
		}
	}
	//2 3
	else if (time_signature[1] < 47)
	{
		mensural = true;
		common = false;
		beat = 0;
		beat_type = 0;
	}
	//fractions
	else
	{
		mensural = false;
		common = false;
		//2x non-decimal
		if (time_signature[1] == '/')
		{
			if (isdigit(time_signature[3]) == 0)
			{
				beat = atoi(&time_signature[0]);
				beat_type = atoi(&time_signature[2]);
			}
			else
			{
				beat = atoi(&time_signature[0]);
				int i,sum = 0;
				for (i=2; i<4; i++) sum = sum*10+(time_signature[i]-48);
				beat_type = sum;
			}
		}
		//decimal & non-decimal
		else if (isdigit(time_signature[4]) == 0)
		{
			int i,sum = 0;
			for (i=0; i<2; i++) sum = sum*10+(time_signature[i]-48);
			beat = sum;
			beat_type = atoi(&time_signature[3]);
		}
		//2x decimal
		else
		{
			int i,sum = 0;
			for (i=0; i<2; i++) sum = sum*10+(time_signature[i]-48);			
			beat = sum;
			sum = 0;
			for (i=3; i<5; i++) sum = sum*10+(time_signature[i]-48);
			beat_type = sum;
		}
	}
	//erase the time signature in memory to avoid strange behaviours
	memset(time_signature, 0, 5);
	time_signature = NULL;
	return create_rhythm(beat, beat_type, common, mensural);
}

int get_keysig(char *incipit, int offset)
{
	int keysig = 0;
	
	if (incipit[offset] == 'x')
	{
		offset++;
		
		while ((incipit[offset] == 'B') || (incipit[offset] == 'E') || (incipit[offset] == 'A') ||
		(incipit[offset] == 'D') || (incipit[offset] == 'G') || (incipit[offset] == 'C') || (incipit[offset] == 'F'))
		{
			keysig++;
			offset++;
		}		
	}

	else if (incipit[offset] == 'b')
	{
		offset++;
		
		while ((incipit[offset] == 'B') || (incipit[offset] == 'E') || (incipit[offset] == 'A') ||
		(incipit[offset] == 'D') || (incipit[offset] == 'G') || (incipit[offset] == 'C') || (incipit[offset] == 'F'))
		{
			keysig--;
			offset++;		
		}
	}
	
	return keysig;
}

int get_alt_keysig(char *incipit, int offset)
{
	int alt_keysig = 0;
	
	if (incipit[offset] == 'x')
	{
		offset++;
		if (incipit[offset] == '[') offset++;

		while ((incipit[offset] == 'B') || (incipit[offset] == 'E') || (incipit[offset] == 'A') ||
		(incipit[offset] == 'D') || (incipit[offset] == 'G') || (incipit[offset] == 'C') || (incipit[offset] == 'F'))
		{
			alt_keysig++;
			offset++;
			if ((incipit[offset] == '[') || (incipit[offset] == ']')) offset++;
		}		
	}

	else if (incipit[offset] == 'b')
	{
		offset++;
		if (incipit[offset] == '[') offset++;
		
		while ((incipit[offset] == 'B') || (incipit[offset] == 'E') || (incipit[offset] == 'A') ||
		(incipit[offset] == 'D') || (incipit[offset] == 'G') || (incipit[offset] == 'C') || (incipit[offset] == 'F'))
		{
			alt_keysig--;
			offset++;
			if ((incipit[offset] == '[') || (incipit[offset] == ']')) offset++;	
		}
	}
	
	return alt_keysig;
}

//least common multiple
int lcm(int nbr1, int nbr2)
{
	int res;
	if ((nbr1 == 0) || (nbr2 == 0)) return 0;
	for (res = (nbr1 < nbr2) ? nbr2 : nbr1; res < (nbr1 * nbr2); res++)
	{
		if (((res % nbr1) == 0) && ((res % nbr2) == 0)) break;
	}
	return res;
}


/******************************************************************/
/*                                                                */
/*                             OUTPUT                             */
/*                                                                */
/******************************************************************/

void output(measure* first_measure, float divisions, int incipit_id, char incipit_nb[6], bool change_notation, int alt_key_signature)
{
	FILE* outxml;

	measure* current_measure;
	note* current_note;
	attribute* current_attribute;
	
	bool open_attribute,natural,open_tuplet,open_notation,new_alter,gracenote;
	int ts_alter = 0;
	int alter = 0;
	char step;
	char filename[22];
	char measure_alters_s[10];
	int measure_alters_a[10];
	
	current_measure = first_measure;
	open_attribute = false;
	open_notation = false;
	new_alter = false;
	gracenote = false;
	
	memset(filename, 0, 22);
	sprintf(filename,"%d-%s.xml",incipit_id,incipit_nb);
	outxml = fopen(filename,"w");

	//header
	fprintf(outxml,"<?xml version=\"1.0\" standalone=\"no\"?>\n");
	fprintf(outxml,"<!DOCTYPE score-partwise PUBLIC \"-//Recordare//DTD MusicXML 1.0 Partwise//EN\" \"http://www.musicxml.org/dtds/partwise.dtd\">\n");
	fprintf(outxml,"<score-partwise>\n");
	fprintf(outxml,"\t<identification>\n");
	fprintf(outxml,"\t\t<encoding>\n");
	fprintf(outxml,"\t\t\t<software>P2X 2005 for Windows/Linux</software>\n");
	fprintf(outxml,"\t\t</encoding>\n");
	fprintf(outxml,"\t\t<miscellaneous>\n");
	fprintf(outxml,"\t\t\t<miscellaneous-field name=\"RISM CH - Incipit ID\">%d</miscellaneous-field>\n",incipit_id);
	fprintf(outxml,"\t\t\t<miscellaneous-field name=\"RISM CH - Incipit NB\">%s</miscellaneous-field>\n",incipit_nb);
	if (change_notation == false) fprintf(outxml,"\t\t\t<miscellaneous-field name=\"Notation\">Common Western Music Notation</miscellaneous-field>\n");
	else if (change_notation == true) fprintf(outxml,"\t\t\t<miscellaneous-field name=\"Notation\">Mensural Notation</miscellaneous-field>\n");
	if (alt_key_signature != 0) fprintf(outxml,"\t\t\t<miscellaneous-field name=\"Corrected Key Signature\">%d</miscellaneous-field>\n",alt_key_signature);
	fprintf(outxml,"\t\t</miscellaneous>\n");
	fprintf(outxml,"\t</identification>\n");
	fprintf(outxml,"\t<part-list>\n");
	fprintf(outxml,"\t\t<score-part id=\"P1\">\n");
	fprintf(outxml,"\t\t\t<part-name>MusicXML Part</part-name>\n");
	fprintf(outxml,"\t\t</score-part>\n");
	fprintf(outxml,"\t</part-list>\n");
	fprintf(outxml,"\t<part id=\"P1\">\n");

	//start the moulinette
	while(1)
	{
		current_attribute = (*current_measure).measure_attributes.start;
		current_note = (*current_measure).notes.start;
		alter = 0;
		natural = false;
		memset(measure_alters_s, 0, 50);
		memset(measure_alters_a, 0, 50);
		int cntma = 0;
		//measure
		fprintf(outxml,"\t\t<measure number=\"%d\">\n",(*current_measure).number);
		//repeating barline
		current_attribute = (*current_measure).measure_attributes.start;
		while	(current_attribute != NULL)
		{
			if ((*current_attribute).id == 53)
			{
				fprintf(outxml,"\t\t\t<barline location=\"left\">\n");
				fprintf(outxml,"\t\t\t\t<bar-style>heavy-light</bar-style>\n");
				fprintf(outxml,"\t\t\t\t<repeat direction=\"forward\"/>\n");
				fprintf(outxml,"\t\t\t</barline>\n");
				break;
			}
			current_attribute = (*current_attribute).next;	
		}				
		//divisions
		if ((*current_measure).number == 1)
		{
			if (open_attribute == false)
			{
				open_attribute = true;
				fprintf(outxml,"\t\t\t<attributes>\n");
			}
			fprintf(outxml,"\t\t\t\t<divisions>%d</divisions>\n",(int) divisions);
		}
		current_attribute = (*current_measure).measure_attributes.start;
		while	(current_attribute != NULL)
		{
			if ((*current_attribute).id == 1)
			{
				ts_alter = (*current_attribute).value;
				if (open_attribute == false)
				{
					open_attribute = true;
					fprintf(outxml,"\t\t\t<attributes>\n");
				}
				fprintf(outxml,"\t\t\t\t<key>\n");
				fprintf(outxml,"\t\t\t\t\t<fifths>%d</fifths>\n",(*current_attribute).value);
				fprintf(outxml,"\t\t\t\t</key>\n");
				break;
			}
			current_attribute = (*current_attribute).next;
		}
		//time signature
		current_attribute = (*current_measure).measure_attributes.start;
		if ((*current_measure).time_signature != NULL)
		{
			if (open_attribute == false)
			{
				open_attribute = true;
				fprintf(outxml,"\t\t\t<attributes>\n");
			}
			if ((*(*current_measure).time_signature).common == true)
			{
				if ((*(*current_measure).time_signature).beat == 2) fprintf(outxml,"\t\t\t\t<time symbol=\"cut\">\n");
				else if ((*(*current_measure).time_signature).beat == 4) fprintf(outxml,"\t\t\t\t<time symbol=\"common\">\n");
				fprintf(outxml,"\t\t\t\t\t<beats>%d</beats>\n",(*(*current_measure).time_signature).beat);
				fprintf(outxml,"\t\t\t\t\t<beat-type>%d</beat-type>\n",(*(*current_measure).time_signature).beat_type);
				fprintf(outxml,"\t\t\t\t</time>\n");
			}
			else
			{
				fprintf(outxml,"\t\t\t\t<time>\n");
				fprintf(outxml,"\t\t\t\t\t<beats>%d</beats>\n",(*(*current_measure).time_signature).beat);
				fprintf(outxml,"\t\t\t\t\t<beat-type>%d</beat-type>\n",(*(*current_measure).time_signature).beat_type);
				fprintf(outxml,"\t\t\t\t</time>\n");
			}
		}
		while	(current_attribute != NULL)
		{
			//clefs
			if (((*current_attribute).id > 9) && ((*current_attribute).id < 14))
			{
				if (open_attribute == false)
				{
					open_attribute = true;
					fprintf(outxml,"\t\t\t<attributes>\n");
				}	
				fprintf(outxml,"\t\t\t\t<clef>\n");
				if ((*current_attribute).id == 10)
				{
					fprintf(outxml,"\t\t\t\t\t<sign>G</sign>\n");
					fprintf(outxml,"\t\t\t\t\t<line>%d</line>\n",(*current_attribute).value);
				}
				else if ((*current_attribute).id == 11)
				{
					fprintf(outxml,"\t\t\t\t\t<sign>F</sign>\n");
					fprintf(outxml,"\t\t\t\t\t<line>%d</line>\n",(*current_attribute).value);
				}
				else if ((*current_attribute).id == 12)
				{
					fprintf(outxml,"\t\t\t\t\t<sign>C</sign>\n");
					fprintf(outxml,"\t\t\t\t\t<line>%d</line>\n",(*current_attribute).value);
				}
				else if ((*current_attribute).id == 13)
				{
					fprintf(outxml,"\t\t\t\t\t<sign>G</sign>\n");
					fprintf(outxml,"\t\t\t\t\t<line>%d</line>\n",(*current_attribute).value);
					fprintf(outxml,"\t\t\t\t\t<clef-octave-change>-1</clef-octave-change>");
				}
				fprintf(outxml,"\t\t\t\t</clef>\n");
			}
			//measure rests
			else if ((*current_attribute).id == 3)
			{
				if (open_attribute == false)
				{
					open_attribute = true;
					fprintf(outxml,"\t\t\t<attributes>\n");
				}
				fprintf(outxml,"\t\t\t\t<measure-style>\n");
				fprintf(outxml,"\t\t\t\t\t<multiple-rest>%d</multiple-rest>\n",(*current_attribute).value);
				fprintf(outxml,"\t\t\t\t</measure-style>\n");
			}
			current_attribute = (*current_attribute).next;
		}
		
		if (open_attribute == true)
		{
			open_attribute = false;
			fprintf(outxml,"\t\t\t</attributes>\n");
		}
		
		//note
		while (current_note != NULL)
		{
			gracenote = false;
			current_attribute = (*current_note).note_attributes.start;
			fprintf(outxml,"\t\t\t<note>\n");
			
			//graces
			while (current_attribute != NULL)
			{
				//slashed
				if ((*current_attribute).id == 24)
				{
					fprintf(outxml,"\t\t\t\t<grace slash=\"yes\"/>\n");
					gracenote = true;
					break;
				}
				//unslashed
				else if ((*current_attribute).id == 25)
				{
					fprintf(outxml,"\t\t\t\t<grace/>\n");
					gracenote = true;
					break;
				}
				current_attribute = (*current_attribute).next;
			}
			current_attribute = (*current_note).note_attributes.start;
			//rests
			if ((*current_note).step == 'R')	fprintf(outxml,"\t\t\t\t<rest/>\n");			
			//note itself
			else
			{
				fprintf(outxml,"\t\t\t\t<pitch>\n");
				fprintf(outxml,"\t\t\t\t\t<step>%c</step>\n",(*current_note).step);
				
				//accidental alters
				while (current_attribute != NULL)
				{
					if ((*current_attribute).id == 21)
					{
						if ((*current_attribute).value == 1)
						{
							int fx;
							bool fixed = false;
							for (fx = 0; fx < cntma; fx++)
							{
								if (measure_alters_s[fx] == (*current_note).step) measure_alters_a[fx] = 0;
								fixed = true;
							}
							if (fixed == false)
							{
								measure_alters_s[cntma] = (*current_note).step;
								measure_alters_a[cntma] = 0;
								cntma++;
							}
							
							break;
						}
						else if ((*current_attribute).value == 2)
						{
							fprintf(outxml,"\t\t\t\t\t<alter>1</alter>\n");
							new_alter = true;
							bool fixed = false;
							int fx;
							for (fx = 0; fx < cntma; fx++)
							{
								if (measure_alters_s[fx] == (*current_note).step) measure_alters_a[fx] = 1;
								fixed = true;
							}
							if (fixed == false)
							{
								measure_alters_s[cntma] = (*current_note).step;
								measure_alters_a[cntma] = 1;
								cntma++;
							}
							break;
						}
						else if ((*current_attribute).value == 3)
						{
							fprintf(outxml,"\t\t\t\t\t<alter>2</alter>\n");
							new_alter = true;
							bool fixed = false;
							int fx;
							for (fx = 0; fx < cntma; fx++)
							{
								if (measure_alters_s[fx] == (*current_note).step) measure_alters_a[fx] = 2;
								fixed = true;
							}
							if (fixed == false)
							{
								measure_alters_s[cntma] = (*current_note).step;
								measure_alters_a[cntma] = 2;
								cntma++;
							}
							break;
						}
						else if ((*current_attribute).value == 4)
						{
							fprintf(outxml,"\t\t\t\t\t<alter>-1</alter>\n");
							new_alter = true;
							bool fixed = false;
							int fx;
							for (fx = 0; fx < cntma; fx++)
							{
								if (measure_alters_s[fx] == (*current_note).step) measure_alters_a[fx] = -1;
								fixed = true;
							}
							if (fixed == false)
							{
								measure_alters_s[cntma] = (*current_note).step;
								measure_alters_a[cntma] = -1;
								cntma++;
							}
							break;
						}
						else if ((*current_attribute).value == 5)
						{
							fprintf(outxml,"\t\t\t\t\t<alter>-2</alter>\n");
							new_alter = true;
							bool fixed = false;
							int fx;
							for (fx = 0; fx < cntma; fx++)
							{
								if (measure_alters_s[fx] == (*current_note).step) measure_alters_a[fx] = -2;
								fixed = true;
							}
							if (fixed == false)
							{
								measure_alters_s[cntma] = (*current_note).step;
								measure_alters_a[cntma] = -2;
								cntma++;
							}
							break;
						}
					}
					current_attribute = (*current_attribute).next;
				}
				
				int alt;
				for (alt = 0; alt < cntma; alt++)
				{
					if ((measure_alters_s[alt] == (*current_note).step) && (new_alter == false))
					{
						if (measure_alters_a[alt] != 0)
						{
							fprintf(outxml,"\t\t\t\t\t<alter>%d</alter>\n",measure_alters_a[alt]);
						}
					}
				}

				new_alter = false;

				current_attribute = (*current_note).note_attributes.start;
				if (ts_alter != 0)
				{
					//natural = false;
					while (current_attribute != NULL)
					{
						if ((*current_attribute).id == 21)
						{
							natural = true;
							break;
						}
						current_attribute = (*current_attribute).next;
					}
					
					alt = 0;
					for (alt = 0; alt < cntma; alt++)
					{
						if (measure_alters_s[alt] == (*current_note).step)
						{
							natural = true;
						}
					}					
					
					if (natural == false)
					{
						if ((*current_note).step == 'F' && ts_alter >= 1) fprintf(outxml,"\t\t\t\t\t<alter>1</alter>\n");
						else if ((*current_note).step == 'C' && ts_alter >= 2) fprintf(outxml,"\t\t\t\t\t<alter>1</alter>\n");
						else if ((*current_note).step == 'G' && ts_alter >= 3) fprintf(outxml,"\t\t\t\t\t<alter>1</alter>\n");
						else if ((*current_note).step == 'D' && ts_alter >= 4) fprintf(outxml,"\t\t\t\t\t<alter>1</alter>\n");
						else if ((*current_note).step == 'A' && ts_alter >= 5) fprintf(outxml,"\t\t\t\t\t<alter>1</alter>\n");
						else if ((*current_note).step == 'E' && ts_alter >= 6) fprintf(outxml,"\t\t\t\t\t<alter>1</alter>\n");
						else if ((*current_note).step == 'B' && ts_alter >= 7) fprintf(outxml,"\t\t\t\t\t<alter>1</alter>\n");
						else if ((*current_note).step == 'B' && ts_alter <= -1) fprintf(outxml,"\t\t\t\t\t<alter>-1</alter>\n");
						else if ((*current_note).step == 'E' && ts_alter <= -2) fprintf(outxml,"\t\t\t\t\t<alter>-1</alter>\n");
						else if ((*current_note).step == 'A' && ts_alter <= -3) fprintf(outxml,"\t\t\t\t\t<alter>-1</alter>\n");
						else if ((*current_note).step == 'D' && ts_alter <= -4) fprintf(outxml,"\t\t\t\t\t<alter>-1</alter>\n");
						else if ((*current_note).step == 'G' && ts_alter <= -5) fprintf(outxml,"\t\t\t\t\t<alter>-1</alter>\n");
						else if ((*current_note).step == 'C' && ts_alter <= -6) fprintf(outxml,"\t\t\t\t\t<alter>-1</alter>\n");
						else if ((*current_note).step == 'F' && ts_alter <= -7) fprintf(outxml,"\t\t\t\t\t<alter>-1</alter>\n");						
					}
					natural = false;
				}
				//octave
				if ((*current_note).octave > 0) fprintf(outxml,"\t\t\t\t\t<octave>%d</octave>\n",((*current_note).octave)+3);
				else if ((*current_note).octave < 0) fprintf(outxml,"\t\t\t\t\t<octave>%d</octave>\n",((*current_note).octave)+4);
				fprintf(outxml,"\t\t\t\t</pitch>\n");
			}
			
			//duration
			if (gracenote == false)
			{
				current_attribute = (*current_note).note_attributes.start;
				while (current_attribute != NULL)
				{
					//dots changes
					if ((*current_attribute).id == 20)
					{
						if ((*current_attribute).value == 1)
						{
							fprintf(outxml,"\t\t\t\t<duration>%d</duration>\n",(int) (((*current_note).rhythm)*divisions*1.5f));
							break;
						}
						else if ((*current_attribute).value == 2)
						{
							fprintf(outxml,"\t\t\t\t<duration>%d</duration>\n",(int) (((*current_note).rhythm)*divisions*1.75f));
							break;
						}
					}
					//tuplets changes
					else if (((*current_attribute).id >= 30) && ((*current_attribute).id <= 32))
					{
						int nn = 31;
						while(!((1<<nn)&((*current_attribute).value))) nn--;
						fprintf(outxml,"\t\t\t\t<duration>%d</duration>\n",(int) ((((*current_note).rhythm)*divisions*(1<<nn))/((*current_attribute).value)));
						break;
					}
					
					if ((current_attribute = (*current_attribute).next) == NULL) fprintf(outxml,"\t\t\t\t<duration>%d</duration>\n",(int) (((*current_note).rhythm)*divisions));
				}
				current_attribute = (*current_note).note_attributes.start;
				if (current_attribute == NULL) fprintf(outxml,"\t\t\t\t<duration>%d</duration>\n",(int) (((*current_note).rhythm)*divisions));
			}
			
			//voice
			fprintf(outxml,"\t\t\t\t<voice>1</voice>\n");
			
			//type
			if (((*current_note).step != 'R') && ((*current_note).rhythm != 4))
			{
				bool ok = false;
				if (gracenote == true)
				{
					current_attribute = (*current_note).note_attributes.start;
					while (current_attribute != NULL)
					{
						if ((*current_attribute).id == 24)	
						{
							fprintf(outxml,"\t\t\t\t<type>eighth</type>\n");
							ok = true;
							break;
						}
						current_attribute = (*current_attribute).next;
					}
				}
		
				//type is it's not a slashed grace note
				if (ok == false)
				{
					if ((*current_note).rhythm == 16.0f) fprintf(outxml,"\t\t\t\t<type>long</type>\n");
					else if ((*current_note).rhythm == 8.0f) fprintf(outxml,"\t\t\t\t<type>breve</type>\n");
					else if ((*current_note).rhythm == 4.0f) fprintf(outxml,"\t\t\t\t<type>whole</type>\n");
					else if ((*current_note).rhythm == 2.0f) fprintf(outxml,"\t\t\t\t<type>half</type>\n");
					else if ((*current_note).rhythm == 1.0f) fprintf(outxml,"\t\t\t\t<type>quarter</type>\n");
					else if ((*current_note).rhythm == 0.5f) fprintf(outxml,"\t\t\t\t<type>eighth</type>\n");
					else if ((*current_note).rhythm == 0.25f) fprintf(outxml,"\t\t\t\t<type>16th</type>\n");
					else if ((*current_note).rhythm == 0.125f) fprintf(outxml,"\t\t\t\t<type>32nd</type>\n");
					else if ((*current_note).rhythm == 0.0625f) fprintf(outxml,"\t\t\t\t<type>64th</type>\n");
					else if ((*current_note).rhythm == 0.03125f) fprintf(outxml,"\t\t\t\t<type>128th</type>\n");
				}
			}
			
			//attributes
			current_attribute = (*current_note).note_attributes.start;
			while (current_attribute != NULL)
			{
				//dotted note
				if ((*current_attribute).id == 20)
				{
					(*current_attribute).value == 2 ? fprintf(outxml,"\t\t\t\t<dot/>\n\t\t\t\t<dot/>\n") : fprintf(outxml,"\t\t\t\t<dot/>\n");
				}

				//accidental
				else if ((*current_attribute).id == 21)
				{
					if ((*current_attribute).value == 1) fprintf(outxml,"\t\t\t\t<accidental>natural</accidental>\n");
					else if ((*current_attribute).value == 2) fprintf(outxml,"\t\t\t\t<accidental>sharp</accidental>\n");
					else if ((*current_attribute).value == 3) fprintf(outxml,"\t\t\t\t<accidental>double-sharp</accidental>\n");
					else if ((*current_attribute).value == 4) fprintf(outxml,"\t\t\t\t<accidental>flat</accidental>\n");
					else if ((*current_attribute).value == 5) fprintf(outxml,"\t\t\t\t<accidental>double-flat</accidental>\n");
				}
				current_attribute = (*current_attribute).next;
			}

			//time modifications
			current_attribute = (*current_note).note_attributes.start;
			while (current_attribute != NULL)
			{
				if (((*current_attribute).id >= 30) && ((*current_attribute).id <= 32))
				{
					fprintf(outxml,"\t\t\t\t<time-modification>\n");
					fprintf(outxml,"\t\t\t\t\t<actual-notes>%d</actual-notes>\n",(*current_attribute).value);
					int nn = 31;
					while(!((1<<nn)&((*current_attribute).value))) nn--;
					fprintf(outxml,"\t\t\t\t\t<normal-notes>%d</normal-notes>\n",1<<nn);
					fprintf(outxml,"\t\t\t\t</time-modification>\n");
				}
				current_attribute = (*current_attribute).next;
			}
			
			//stem up if grace
			current_attribute = (*current_note).note_attributes.start;
			while (current_attribute != NULL)
			{
				if (((*current_attribute).id == 24) || ((*current_attribute).id == 25))
				{
					fprintf(outxml,"\t\t\t\t<stem>up</stem>\n");
				}
				current_attribute = (*current_attribute).next;
			}			
			
			//beam
			current_attribute = (*current_note).note_attributes.start;
			while (current_attribute != NULL)
			{
				if ((*current_attribute).id == 40) fprintf(outxml,"\t\t\t\t<beam number=\"1\">begin</beam>\n");
				else if ((*current_attribute).id == 41) fprintf(outxml,"\t\t\t\t<beam number=\"1\">continue</beam>\n");
				else if ((*current_attribute).id == 42) fprintf(outxml,"\t\t\t\t<beam number=\"1\">end</beam>\n");
				current_attribute = (*current_attribute).next;
			}

			//notations
			current_attribute = (*current_note).note_attributes.start;
			while (current_attribute != NULL)
			{
				//tuplets
				if ((*current_attribute).id == 30)
				{
					if (open_notation == false)
					{
						open_notation = true;
						fprintf(outxml,"\t\t\t\t<notations>\n");
					}
					fprintf(outxml,"\t\t\t\t\t<tuplet type=\"start\" placement=\"above\" bracket=\"yes\"/>\n");
				}
				else if ((*current_attribute).id == 32)
				{
					if (open_notation == false)
					{
						open_notation = true;
						fprintf(outxml,"\t\t\t\t<notations>\n");
					}
					fprintf(outxml,"\t\t\t\t\t<tuplet type=\"stop\"/>\n");
				}
				//fermatas
				else if ((*current_attribute).id == 22)
				{
					if (open_notation == false)
					{
						open_notation = true;
						fprintf(outxml,"\t\t\t\t<notations>\n");
					}
					fprintf(outxml,"\t\t\t\t\t<fermata type=\"upright\"/>\n");
				}
				//trills
				else if ((*current_attribute).id == 23)
				{
					if (open_notation == false)
					{
						open_notation = true;
						fprintf(outxml,"\t\t\t\t<notations>\n");
					}
					fprintf(outxml,"\t\t\t\t\t<ornaments>\n");
					fprintf(outxml,"\t\t\t\t\t\t<trill-mark/>\n");
					fprintf(outxml,"\t\t\t\t\t</ornaments>\n");
				}
				//slurs
				else if ((*current_attribute).id == 26)
				{
					if (open_notation == false)
					{
						open_notation = true;
						fprintf(outxml,"\t\t\t\t<notations>\n");
					}
					fprintf(outxml,"\t\t\t\t\t<slur type=\"start\" number=\"1\"/>\n");
				}
				else if ((*current_attribute).id == 27)
				{
					if (open_notation == false)
					{
						open_notation = true;
						fprintf(outxml,"\t\t\t\t<notations>\n");
					}
					fprintf(outxml,"\t\t\t\t\t<slur type=\"stop\" number=\"1\"/>\n");
				}
				current_attribute = (*current_attribute).next;
			}
			
			if (open_notation == true)
			{
				open_notation = false;
				fprintf(outxml,"\t\t\t\t</notations>\n");
			}

			//note end
			if ((*current_note).next == NULL)
			{
				fprintf(outxml,"\t\t\t</note>\n");
				break;
			}
			else
			{
				fprintf(outxml,"\t\t\t</note>\n");
				current_note = (*current_note).next;
			}
		} //while note
		
		//measure end
		current_attribute = (*current_measure).measure_attributes.start;
		while	(current_attribute != NULL)
		{
			if ((*current_attribute).id == 50)
			{
				fprintf(outxml,"\t\t\t<barline location=\"right\">\n");
				fprintf(outxml,"\t\t\t\t<bar-style>light-light</bar-style>\n");
				fprintf(outxml,"\t\t\t</barline>\n");
				break;
			}
			if ((*current_attribute).id == 51)
			{
				fprintf(outxml,"\t\t\t<barline location=\"right\">\n");
				fprintf(outxml,"\t\t\t\t<bar-style>light-heavy</bar-style>\n");
				fprintf(outxml,"\t\t\t</barline>\n");
				break;
			}
			if ((*current_attribute).id == 52)
			{
				fprintf(outxml,"\t\t\t<barline location=\"right\">\n");
				fprintf(outxml,"\t\t\t\t<bar-style>light-heavy</bar-style>\n");
				fprintf(outxml,"\t\t\t\t<repeat direction=\"backward\"/>\n");
				fprintf(outxml,"\t\t\t</barline>\n");
				break;
			}						
			current_attribute = (*current_attribute).next;	
		}
		if ((*current_measure).next == NULL)
		{
			fprintf(outxml,"\t\t</measure>\n");
			break;
		}
		else
		{
			fprintf(outxml,"\t\t</measure>\n");
			current_measure = (*current_measure).next;
		}
	} //while measure
	
	fprintf(outxml,"\t</part>\n");
	fprintf(outxml,"</score-partwise>\n");
	
	fclose(outxml);

	current_note = NULL;
	current_attribute = NULL;

	destroy_measure(first_measure);
	first_measure = NULL;
}

/******************************************************************/
/*                                                                */
/*                           CONVERSION                           */
/*                                                                */
/******************************************************************/

void convert_incipit(char* incipit, long size, bool mensural_remove, bool meas_rest_remove, bool verbose, bool xml)
{
	measure* first_measure;
	measure* last_measure;

	int measure_number;
	char current_char;
	char current_symbol;
	char current_step;
	int current_octave;
	float current_rhythmic_value;
	char clef[3];
	int key_signature;
	int alt_key_signature;
	char time_signature[5];
	measure* temp_measure;
	rhythm* ts_pointer;
	int octave_cnt;
	int dot;
	int accidental;
	int rest;
	char temp_rest[2];
	char key_line;
	int shortest_note;
	int tuplet;
	bool new_tuplet,new_beaming,beaming,fermata,acciaccatura,appoggiatura,slur,repeat_sign,rhytmic_model,del_model,error,ms_not;
	int model_cnt;
	int model_size;
	int mult_appo;

	float divisions = 2.0f;
	int incipit_id = 0;
	char incipit_nb[8];

	float* r_model;
	int* d_model;
	
	int i,j,k,z;
	
	first_measure = NULL;
	last_measure = NULL;
	temp_measure = NULL;
	ts_pointer = NULL;
	
	
	i = 0;
	key_signature = 0;
	alt_key_signature = 0;
	current_octave = 1;
	new_tuplet = false;
	new_beaming = false;
	beaming = false;
	fermata = false;
	acciaccatura = false;
	appoggiatura = false;
	slur = false;
	repeat_sign = false;
	rhytmic_model = false;
	ms_not = false;
	del_model = false;
	error = false;
	mult_appo = 0;
	tuplet = 0;
	dot = 0;
	rest = 0;
	accidental = 0;
	shortest_note = 0;
	model_cnt = 0;
	model_size = 0;


	//end of file
	if (incipit[i] == '\n' || incipit[i] == '\xD')
	{
		fprintf(stdout,"\nEOF, quitting\n");
		exit(0);
	}
	//consum
	while (isdigit(incipit[i]) == 0) i++;
	
	//incipit id
	int id_cnt;
	incipit_id = 0;
	for (id_cnt=1; id_cnt<10; id_cnt++)
	{
		incipit_id = incipit_id*10+(incipit[i]-48);
		i++;
	}
	
	while (isdigit(incipit[i]) == 0) i++;

	int nb_cnt;
	memset(incipit_nb, 0, 8);
	for (nb_cnt = 0; nb_cnt < 8; nb_cnt++)
	{
		incipit_nb[nb_cnt] = incipit[i];
		i++;
		if (incipit[i] == '\t')	break;
	}
	
	while (incipit[i] == '\t') i++;
	
	//the clef is always there and always 3 chars long
	for (j=0; j<3; j++) clef[j] = incipit[j+i];
	if (clef[1] == '+') ms_not = true;
	i=i+3;

	while (incipit[i] == '\t') i++;

	//time signature
	if ((isdigit(incipit[i])) || (incipit[i] == 'c') || (incipit[i] == 'o'))
	{
		j=0;
		while (incipit[i] != '\t')
		{
			time_signature[j] = incipit[i];
			i++;
			j++;
		}
		ts_pointer = rhythm_convert((void *)&time_signature);
		if ((*ts_pointer).mensural == true)
		{
			i++;
			j=0;
			memset(time_signature, 0, 5);
			if (((isdigit(incipit[i])) && (incipit[i] !='0')) || (incipit[i] == 'c') || (incipit[i] == 'o'))
			{
				while (incipit[i] != '\t')
				{
					time_signature[j] = incipit[i];
					i++;
					j++;
				}
				free(ts_pointer);
				ts_pointer = NULL;
				ts_pointer = rhythm_convert((void *)&time_signature);
				(*ts_pointer).mensural = true;
			}
			else if ((incipit[i] == '\\') || (incipit[i] =='0'))
			{
				while (incipit[i] != '\t') i++;
				ms_not = true;				
			}
		}
		else
		{
			i++;
			while (incipit[i] != '\t') i++;
		}
		memset(time_signature, 0, 5);
	}
	else if (incipit[i] == '\\')
	{
		i = i+3;
		while (incipit[i] != '\t') i++;
	}

	if ((ms_not == true) && (ts_pointer != NULL))
	{
			free(ts_pointer);
			ts_pointer = NULL;
	}

	if (ts_pointer != NULL)
	{
		if ((*ts_pointer).mensural == true) ms_not = true;
	}

	if ((mensural_remove == true) && (ts_pointer != NULL))
	{
		if ((*ts_pointer).mensural == true)
		{
			free(ts_pointer);
			ts_pointer = NULL;
		}
	}

	//key signature
	while (incipit[i] == '\t') i++;

	if (incipit[i] == '[')
	{
		i++;
		printf("poop\n");
		if (incipit[i] == '$') i++;
		alt_key_signature = get_alt_keysig(incipit,i);
		fprintf(stderr,"\nWARNING,\n incipit: %d-%s, removing time signature within []\n...outputting to log file\n\n",incipit_id,incipit_nb);
		fprintf(errlog,"\nWARNING,\n incipit: %d-%s, removing time signature within []\n",incipit_id,incipit_nb);
		while (incipit[i] != ']') i++;
		i++;
	}

	else if (incipit[i] == '$')
	{
		i++;
		if (incipit[i] != '[')
		{
			key_signature = get_keysig(incipit,i);
			i = i+abs(key_signature)+1;
			if (incipit[i] == '[')
			{
				i = i-abs(key_signature)-1;
				alt_key_signature = get_alt_keysig(incipit,i);
				while (incipit[i] != ']') i++;
				i++;
				fprintf(stderr,"\nWARNING,\n incipit: %d-%s, removing time signature within []\n...outputting to log file\n\n",incipit_id,incipit_nb);
				fprintf(errlog,"\nWARNING,\n incipit: %d-%s, removing time signature within []\n",incipit_id,incipit_nb);
			}
		}
		else if (incipit[i] == '[')
		{
			i++;
			alt_key_signature = get_alt_keysig(incipit,i);
			while (incipit[i] != ']') i++;
			i++;
			fprintf(stderr,"\nWARNING,\n incipit: %d-%s, removing time signature within []\n...outputting to log file\n\n",incipit_id,incipit_nb);
			fprintf(errlog,"\nWARNING,\n incipit: %d-%s, removing time signature within []\n",incipit_id,incipit_nb);
		}
	}
	
	//start of context, just pass over it
	while (incipit[i] == ' ' || incipit[i] == '&' || incipit[i] == '' || incipit[i] == '' || incipit[i] == '') i++;
	
	//creation of the data structure
	measure_number = 1;
	first_measure = create_measure(measure_number, NULL);
	last_measure = first_measure;

	//add the clef to the first measure
	if (clef[0] == 'G')
	{
		attribute* temp_attrib = create_attribute(10, atoi(&clef[2]), NULL);
		add_measure_attrib(temp_attrib,last_measure);
		temp_attrib = NULL;
	}
	else if (clef[0] == 'F')
	{
		attribute* temp_attrib = create_attribute(11, atoi(&clef[2]), NULL);
		add_measure_attrib(temp_attrib,last_measure);
		temp_attrib = NULL;
	}
	else if (clef[0] == 'C')
	{
		attribute* temp_attrib = create_attribute(12, atoi(&clef[2]), NULL);
		add_measure_attrib(temp_attrib,last_measure);
		temp_attrib = NULL;
	}
	else if (clef[0] == 'g') 
	{
		attribute* temp_attrib = create_attribute(13, atoi(&clef[2]), NULL);
		add_measure_attrib(temp_attrib,last_measure);
		temp_attrib = NULL;
	}
	memset(clef, 0, 3);
	
	//add the key signature to the first measure
	if (key_signature != 0)
	{
		attribute* temp_attrib = create_attribute(1,key_signature, NULL);
		add_measure_attrib(temp_attrib,last_measure);
		temp_attrib = NULL;
		key_signature = 0;
	}
	
	//add the time signature to the first measure;
	if (ts_pointer != NULL)
	{
		(*last_measure).time_signature = ts_pointer;
		memset(time_signature, 0, 5);
		ts_pointer = NULL;
	}
	
	//now for real
	while(i < size)
	{
		if (incipit[i] == ' ') i++;

		//positive octave
		else if (incipit[i] == '\'')
		{
			octave_cnt = 1;
			i++;
			while (incipit[i] == '\'')
			{
				octave_cnt++;
				i++;
			}
			current_octave = octave_cnt;
		}
		//negative octave
		else if (incipit[i] == ',')
		{
			octave_cnt = -1;
			i++;
			while (incipit[i] == ',')
			{
				octave_cnt--;
				i++;
			}
			current_octave = octave_cnt;
		}

		//rhythmic value
		else if (isdigit(incipit[i]) != 0)
		{
			//tricky trick in case of a rhytmic model
			char tricktheatoi[2];
			memset(tricktheatoi, 0, 2);
			tricktheatoi[0] = incipit[i];
			
			int temp_rv = atoi(&tricktheatoi[0]);
			
			if (temp_rv == 0) current_rhythmic_value = 16.0f;
			else if (temp_rv == 9) current_rhythmic_value = 8.0f;
			else if (temp_rv == 1) current_rhythmic_value = 4.0f;
			else if (temp_rv == 2) current_rhythmic_value = 2.0f;
			else if (temp_rv == 4) current_rhythmic_value = 1.0f;
			else if (temp_rv == 8) current_rhythmic_value = 0.5f;
			else if (temp_rv == 6) current_rhythmic_value = 0.25f;
			else if (temp_rv == 3) current_rhythmic_value = 0.125f;
			else if (temp_rv == 5) current_rhythmic_value = 0.0625f;
			else if (temp_rv == 7) current_rhythmic_value = 0.03125f;

			dot=0;
			i++;
			
			while (incipit[i] == '.')
			{
				dot++;
				i++;
			}
			
			if ((dot == 1) && (temp_rv == 7))
			{
				current_rhythmic_value = 1;
				dot = 0;
				fprintf(stderr,"\nWARNING,\n incipit: %d-%s, found a note in neumatic notation (7.), using quarter note instead\n...outputting to log file\n\n",incipit_id,incipit_nb);
				fprintf(errlog,"\nWARNING,\n incipit: %d-%s, found a note in neumatic notation (7.), using quarter note instead\n",incipit_id,incipit_nb);					
			}
			
			if (dot == 1)
			{
				if ((int) (1.0f/(current_rhythmic_value*1.5f)) > 0) divisions = (float) lcm((int) divisions, (int) ((1.0f/(current_rhythmic_value*1.5f))));
			}
			else if (dot == 2)
			{
				if ((int) (1.0f/(current_rhythmic_value*1.75f)) > 0) divisions = (float) lcm((int) divisions, (int) ((1.0f/(current_rhythmic_value*1.75f))));
			}
			else
			{
				if ((int) (1.0f/current_rhythmic_value) > 0) divisions = (float) lcm((int) divisions, (int) ((1.0f/current_rhythmic_value)));
			}
			
			//rhytmic model
			if (del_model == true)
			{
					free(r_model);
					free(d_model);
					r_model = NULL;
					d_model = NULL;
					del_model = false;
					rhytmic_model = false;
			}
			
			if ((rhytmic_model == false) && (isdigit(incipit[i]) != 0))
			{
				rhytmic_model = true;
				r_model = (float*)malloc(sizeof(float)*255);
				d_model = (int*)malloc(sizeof(int)*255);
				r_model[model_cnt] = current_rhythmic_value;
				d_model[model_cnt] = dot;
			}
			
			
			else if (rhytmic_model == true)
			{
				r_model[model_cnt] = current_rhythmic_value;
				d_model[model_cnt] = dot;
			}
			
			if (rhytmic_model == true)
			{
				model_cnt++;
				model_size = model_cnt;
			}
		}
	
		//accidentals (1 = n; 2 = x; 3 = xx; 4 = b; 5 = bb)
		else if (incipit[i] == 'n' || incipit[i] == 'x' || incipit[i] == 'b')
		{
			if (incipit[i] == 'n')
			{
				accidental = 1;
				i++;
			}
			else if (incipit[i] == 'x')
			{
				accidental = 2;
				i++;
				if (incipit[i] == 'x')
				{
					accidental++;
					i++;
				}
			}
			else if (incipit[i] == 'b')
			{
				accidental = 4;
				i++;
				if (incipit[i] == 'b')
				{
					accidental++;
					i++;
				}
			}
		}
		
		//beamings
		else if (incipit[i] == '{')
		{
			new_beaming = true;
			beaming = true;
			i++;
		}
		
		else if (incipit[i] == '}')
		{
			beaming = false;
			i++;
		}
		
		else if (incipit[i] == '+') i++;
		
		//tuplets & fermatas
		else if (incipit[i] == '(')
		{
			//fermata
			if ((incipit[i+2] == ')') || (incipit[i+3] == ')'))
			{
				fermata = true;
				i++;
			}
			
			//tuplets
			else
			{
				int t = i;
				while (incipit[t] != ')') t++;
				
				//triplet not ending with ";3)"
				if (isdigit(incipit[t-1]) == 0) tuplet = 3;
				//any other tuplet
				else
				{
					tuplet = atoi(&incipit[t-1]);
					if (incipit[t-2] != ';') tuplet = 10+tuplet;
				}
				divisions = (float) lcm((int) divisions, tuplet);
				new_tuplet = true;
				i++;
			}
		}
		
		else if (incipit[i] == ';')
		{
			tuplet = 0;
			while (incipit[i] != ')') i++;
			i++;
		}
			
		else if (incipit[i] == ')')
		{
			if ((incipit[i-2] == '(') || (incipit[i-3] == '('))
			{
				fermata = false;
				i++;
			}
			else
			{
				tuplet = 0;
				i++;
			}
		}
		
		//trill after a fermata, special case to ignore, all other trills are treated afterwards
		else if (incipit[i] == 't') i++;

		//grace notes
		//acciaccatura
		else if (incipit[i] == 'g')
		{
			acciaccatura = true;
			i++;
			
			if (del_model == true)
			{
					free(r_model);
					free(d_model);
					r_model = NULL;
					d_model = NULL;
					del_model = false;
					rhytmic_model = false;
			}			
			
		}
		
		//appoggiatura
		else if (incipit[i] == 'q')
		{
			appoggiatura = true;
			i++;

			if (del_model == true)
			{
					free(r_model);
					free(d_model);
					r_model = NULL;
					d_model = NULL;
					del_model = false;
					rhytmic_model = false;
			}

			if (incipit[i] == 'q')
			{
				int r = i;
				while (incipit[r] != 'r')
				{
					if (incipit[r] == 'A' || incipit[r] == 'B' || incipit[r] == 'C' || incipit[r] == 'D' || 
						 incipit[r] == 'E' || incipit[r] == 'F' || incipit[r] == 'G')
					{
						mult_appo++;
					}
					r++;
				}
				i++;
			}
		}
		
		else if (incipit[i] == 'r')
		{
			appoggiatura = false;
			mult_appo = 0;
			i++;
		}
		
		//repetitions
		else if (incipit[i] == '!') i++;
		else if (incipit[i] == 'f')
		{
			int replen = 0;
			int repoffset = i;
			while (incipit[repoffset] != '!') repoffset--;
			repoffset--;
			while (incipit[repoffset] != '!') repoffset--;
			incipit[i] = 'z';
			i = i-(i-repoffset);
		}
		else if (incipit[i] == 'z') i++;
		
		//note and rest
		else if (incipit[i] == 'A' || incipit[i] == 'B' || incipit[i] == 'C' || incipit[i] == 'D' ||
					incipit[i] == 'E' || incipit[i] == 'F' || incipit[i] == 'G' || incipit[i] == '-')
		{
			if (incipit[i] == '-') current_step = 'R';
			else current_step = incipit[i];
		
			if (rhytmic_model == true)
			{
				del_model = true;
				current_rhythmic_value = r_model[model_size-model_cnt];
				dot = d_model[model_size-model_cnt];
				if	(model_cnt == 1) model_cnt = model_size;
				else model_cnt--;
			}

			note *temp_note = create_note(current_step, current_octave, current_rhythmic_value, NULL);
			add_note(temp_note,last_measure);
			
			if (dot > 0)
			{
				attribute* tmp_att = create_attribute(20, dot, NULL);
				add_note_attrib(tmp_att,(*last_measure).notes.end);
				tmp_att = NULL;
				dot = 0;
			}
			if (accidental > 0)
			{
				attribute* tmp_att = create_attribute(21, accidental, NULL);
				add_note_attrib(tmp_att,(*last_measure).notes.end);
				tmp_att = NULL;
				accidental = 0;
			}
			if (tuplet > 0)
			{
				if (new_tuplet == true)
				{
					new_tuplet = false;
					attribute* tmp_att = create_attribute(30, tuplet, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
				}
				else if ((incipit[i+1] == ';') || (incipit[i+1] == ')'))
				{
					attribute* tmp_att = create_attribute(32, tuplet, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
				}
				else if ((incipit[i+1] == '}') && ((incipit[i+1] == ')') || (incipit[i+1] == ';')))
				{
					attribute* tmp_att = create_attribute(32, tuplet, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
				}
				else
				{
					attribute* tmp_att = create_attribute(31, tuplet, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
				}

			}
			if (beaming == true)
			{
				if (new_beaming == true)
				{
					new_beaming = false;
					attribute* tmp_att = create_attribute(40, 0, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
				}
				else if (incipit[i+1] == '}')
				{
					attribute* tmp_att2 = create_attribute(42, 0, NULL);
					add_note_attrib(tmp_att2,(*last_measure).notes.end);
					tmp_att2 = NULL;
					if (incipit[i+2] == '+')
					{
						attribute* tmp_att = create_attribute(26, 0, NULL);
						add_note_attrib(tmp_att,(*last_measure).notes.end);
						tmp_att = NULL;
						slur = true;
					}
				}
				else
				{
					attribute* tmp_att = create_attribute(41, 0, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
				}
			}
			
			//fermatas
			if (fermata == true)
			{
				attribute* tmp_att = create_attribute(22, 0, NULL);
				add_note_attrib(tmp_att,(*last_measure).notes.end);
				tmp_att = NULL;
				fermata = false;
				
				//trill after fermata
				if (incipit[i+2] == 't')
				{
					attribute* tmp_att = create_attribute(23, 0, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
					i++;
				}
			}
			
			//trills
			if (incipit[i+1] == 't')
			{
				attribute* tmp_att = create_attribute(23, 0, NULL);
				add_note_attrib(tmp_att,(*last_measure).notes.end);
				tmp_att = NULL;
				i++;
			}
			
			//slurs
			if (incipit[i+1] == '+')
			{
				attribute* tmp_att = create_attribute(26, 0, NULL);
				add_note_attrib(tmp_att,(*last_measure).notes.end);
				tmp_att = NULL;
				slur = true;
			}
			else if ((slur == true) && (incipit[i+1] != '}'))
			{
				attribute* tmp_att = create_attribute(27, 0, NULL);
				add_note_attrib(tmp_att,(*last_measure).notes.end);
				tmp_att = NULL;
				slur = false;
			}
			
			//graces
			if (acciaccatura == true)
			{
				attribute* tmp_att = create_attribute(24, 0, NULL);
				add_note_attrib(tmp_att,(*last_measure).notes.end);
				tmp_att = NULL;
				acciaccatura = false;
			}
			
			if (appoggiatura == true)
			{
				if (mult_appo > 0)
				{
					attribute* tmp_att = create_attribute(25, 0, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
					attribute* tmp_att2 = create_attribute(40, 0, NULL);
					add_note_attrib(tmp_att2,(*last_measure).notes.end);
					tmp_att2 = NULL;
					mult_appo--;
				}
				else
				{
					attribute* tmp_att = create_attribute(25, 0, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
				}
				appoggiatura = false;
			}
			else if (mult_appo > 0)
			{
				if (mult_appo == 1)
				{
					attribute* tmp_att = create_attribute(25, 0, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
					attribute* tmp_att2 = create_attribute(42, 0, NULL);
					add_note_attrib(tmp_att2,(*last_measure).notes.end);
					tmp_att2 = NULL;
				}
				else
				{
					attribute* tmp_att = create_attribute(25, 0, NULL);
					add_note_attrib(tmp_att,(*last_measure).notes.end);
					tmp_att = NULL;
					attribute* tmp_att2 = create_attribute(41, 0, NULL);
					add_note_attrib(tmp_att2,(*last_measure).notes.end);
					tmp_att2 = NULL;
				}
				mult_appo--;
			}

			i++;
		}
		
		//clef change
		else if (incipit[i] == '%')
		{
			i++;
			for (j=0; j<3; j++) clef[j] = incipit[j+i];
			i=i+3;

			if (clef[0] == 'G')
			{
				attribute* temp_attrib = create_attribute(10, atoi(&clef[2]), NULL);
				add_measure_attrib(temp_attrib,last_measure);
				temp_attrib = NULL;
			}
			else if (clef[0] == 'F')
			{
				attribute* temp_attrib = create_attribute(11, atoi(&clef[2]), NULL);
				add_measure_attrib(temp_attrib,last_measure);
				temp_attrib = NULL;
			}
			else if (clef[0] == 'C')
			{
				attribute* temp_attrib = create_attribute(12, atoi(&clef[2]), NULL);
				add_measure_attrib(temp_attrib,last_measure);
				temp_attrib = NULL;
			}
			else if (clef[0] == 'g') 
			{
				attribute* temp_attrib = create_attribute(13, atoi(&clef[2]), NULL);
				add_measure_attrib(temp_attrib,last_measure);
				temp_attrib = NULL;
			}
			memset(clef, 0, 3);
		}
			
		//time signature change
		else if (incipit[i] == '@')
		{
			i++;
			if ((isdigit(incipit[i])) || (incipit[i] == 'c') || (incipit[i] == 'o'))
			{
				j=0;
				char new_ts[5];
				if (incipit[i] == 'c')
				{
					new_ts[0] = 'c';
					i++;
					if (incipit[i] == '/')
					{
						new_ts[1] == '/';
						i++;
					}
				}
				else
				{
					while ((isdigit(incipit[i]) != 0) || (incipit[i] == '/'))
					{
						new_ts[j] = incipit[i];
						i++;
						j++;
						if (j>4) break;
					}
				}				
				(*last_measure).time_signature = rhythm_convert((void *)&new_ts);
				memset(time_signature, 0 ,5);
			}
		}
		
		//key signature change
		else if (incipit[i] == '$')
		{
			i++;
			key_signature = get_keysig(incipit,i);
			i = i+abs(key_signature)+1;
	
			if (key_signature != 0)
			{
				attribute* temp_attrib = create_attribute(1,key_signature, NULL);
				add_measure_attrib(temp_attrib,last_measure);
				temp_attrib = NULL;
				key_signature = 0;
			}			
		}			
		
		//measure rest
		else if (incipit[i] == '=')
		{
			int mr = i;
			while (incipit[mr] != '/') mr++;
			int mrl = mr-i;
			if (meas_rest_remove == false)
			{				
				int cnt,sum;
				sum = 0;
				for (cnt=1; cnt<mrl; cnt++) sum = sum*10+(incipit[i+cnt]-48);

				if (sum > 1)
				{
					attribute* temp_attrib = create_attribute(3, sum, NULL);
					add_measure_attrib(temp_attrib,last_measure);
					temp_attrib = NULL;
				}
	
				note *temp_note = create_note('R', current_octave, 4.0f, NULL);
				add_note(temp_note,last_measure);
				temp_note = NULL;
				
				for (cnt=1; cnt<sum; cnt++)
				{
					temp_measure = create_measure(measure_number+cnt, NULL);
					(*last_measure).next = temp_measure;
					last_measure = (*last_measure).next;
					temp_measure = NULL;
	
					note *temp_note = create_note('R', current_octave, 4.0f, NULL);
					add_note(temp_note,last_measure);
					temp_note = NULL;
				}
				
				if (sum > 0) measure_number = measure_number+sum-1;
				i = i+mrl;
			}
			else
			{
				if ((mrl == 1) || ((mrl == 2) && (incipit[i+1] == '1')))
				{
					note *temp_note = create_note('R', current_octave, 4.0f, NULL);
					add_note(temp_note,last_measure);
					temp_note = NULL;
					i = i+mrl;
				}
				else 
				{
					if (measure_number > 1) i = i+mrl+1;
					else i = i+mrl;
				}
			}
		}
		
		//measure bar repetitions
		else if (incipit[i] == ':')
		{
			repeat_sign = true;
			i++;
		}
		
		//measure end
		else if (incipit[i] == '/' || incipit[i] < 32)
		{
			//incipit end
			if (incipit[i] < 32 || incipit[i+1] < 32) break;
			
			//measure repeat
			else if (incipit[i+1] == 'i')
			{
				measure_number++;
				measure* cp_measure = copy_measure(last_measure);
				(*last_measure).next = cp_measure;
				last_measure = (*last_measure).next;
				cp_measure = NULL;
				i=i+2;
			}
			//measure bar
			else
			{
				if (repeat_sign == true)
				{
					// ://:
					if (incipit[i+2] == ':')
					{
						attribute* temp_attrib = create_attribute(52,0, NULL);
						add_measure_attrib(temp_attrib,last_measure);
						temp_attrib = NULL;
						
						measure_number++;
						temp_measure = create_measure(measure_number, NULL);
						(*last_measure).next = temp_measure;
						last_measure = (*last_measure).next;
						temp_measure = NULL;
						
						attribute* temp_attrib2 = create_attribute(53,0, NULL);
						add_measure_attrib(temp_attrib2,last_measure);
						temp_attrib2 = NULL;
						
						if (incipit[i+3] > 32) i = i+3;
						else break;
					}
					// ://
					else
					{
						attribute* temp_attrib = create_attribute(52,0, NULL);
						add_measure_attrib(temp_attrib,last_measure);
						temp_attrib = NULL;
						
						if (incipit[i+2] > 32) i = i+2;
						else break;
					}
					repeat_sign = false;
				}
				else if (incipit[i+1] == '/')
				{
					// final //
					if (incipit[i+2] < 32)
					{
						attribute* temp_attrib = create_attribute(51,0, NULL);
						add_measure_attrib(temp_attrib,last_measure);
						temp_attrib = NULL;
						break;
					}
					// //:
					if (incipit[i+2] == ':')
					{
						measure_number++;
						temp_measure = create_measure(measure_number, NULL);
						(*last_measure).next = temp_measure;
						last_measure = (*last_measure).next;
						temp_measure = NULL;							
						
						attribute* temp_attrib = create_attribute(53,0, NULL);
						add_measure_attrib(temp_attrib,last_measure);
						temp_attrib = NULL;						
						if (incipit[i+3] > 32) i = i+3;
						else break;
					}
					// normal //
					else
					{
						attribute* temp_attrib = create_attribute(50,0, NULL);
						add_measure_attrib(temp_attrib,last_measure);
						temp_attrib = NULL;
						if (incipit[i+2] > 32)
						{
							i = i+2;
							if (incipit[i] == 'i')
							{
								measure_number++;
								measure* cp_measure = copy_measure(last_measure);
								(*last_measure).next = cp_measure;
								last_measure = (*last_measure).next;
								cp_measure = NULL;
								i=i+1;
							}					
							else
							{
								measure_number++;
								temp_measure = create_measure(measure_number, NULL);
								(*last_measure).next = temp_measure;
								last_measure = (*last_measure).next;
								temp_measure = NULL;
							}
						}
						else break;
					}
				}
				//normal case
				else
				{
					measure_number++;
					temp_measure = create_measure(measure_number, NULL);
					(*last_measure).next = temp_measure;
					last_measure = (*last_measure).next;
					temp_measure = NULL;
					i++;
				}
			}
		}
		else
		{
			error = true;
			fprintf(stderr,"\nERROR,\n incipit: %d-%s, char: %c, ascii: %d, i: %d\n...outputting to log file\n\n",incipit_id,incipit_nb,incipit[i],incipit[i],i);
			fprintf(errlog,"\nERROR,\n incipit: %d-%s, char: %c, ascii: %d, i: %d\n",incipit_id,incipit_nb,incipit[i],incipit[i],i);
			break;
		}
		if (verbose == true) printf("i: %d, char %c, ascii: %d\n",i,incipit[i],incipit[i]);

	}//end while
	
	if (del_model == true)
	{
		free(r_model);
		free(d_model);
		r_model = NULL;
		d_model = NULL;
	}
	
	last_measure = NULL;
	temp_measure = NULL;
	
	if (error == true) destroy_measure(first_measure);
	else if (xml == true) destroy_measure(first_measure);
	else output(first_measure,divisions,incipit_id,incipit_nb,ms_not,alt_key_signature);
	
}


/******************************************************************/
/*                                                                */
/*                              MAIN                              */
/*                                                                */
/******************************************************************/

int main(int argc, char* argv[])
{
	FILE *input;

	if (argc == 1)
	{
		fprintf(stderr,"Try ./p2x2005_10.exe -h for help\n");
		return 1;
  	}
  	
  	char* options = argv[1];
  	int optlen = strlen(options);
  	int optch = 1;
  	bool mensural_remove = false;
  	bool meas_rest_remove = false;
  	bool verbose = false;
  	bool xml = false;
  	
  	while (optch < optlen)
  	{
  		switch (options[optch])
  		{
  			case 'h':
		  		printf("\nP2X 2005 HELP\n");
		  		printf("*************\n\n");
 		 		printf("Usage: ./p2x2005_10.exe -[options]f <input_file>\n\n");
 		 		printf("Available options are:\n");
 		 		printf("m: Remove all mensural time signatures.\n");
 		 		printf("r: Remove all measure rests (useful for image creation).\n");
 		 		printf("v: Verbose mode (shows the progression of each char, not recommended).\n");
 		 		printf("x: Do not create the xml file (useful for debug on a lot of incipits).\n");
 		 		printf("f: Input file in the next argument (necessary).\n");
 		 		printf("h: Show this help.\n");
 		 		return 0;
 		 	case 'm':
 		 		printf("\nOkay, mensural time signatures are removed.\n");
 		 		mensural_remove = true;
 		 		break;
 		 	case 'r':
 		 		printf("\nOkay, measure rests are removed.\n");
 		 		meas_rest_remove = true;
 		 		break;
 		 	case 'v':
 		 		printf("\nOkay, verbose mode enabled.\n");
 		 		verbose = true;
 		 		break;
 		 	case 'x':
 		 		printf("\nOkay, XML file will not be created.\n");
 		 		xml = true;
 		 		break;
 		 	case 'f':
 		 		printf("\n");
 		 		break;
 		 	default:
 		 		printf("\nUnknown option(s), try ./p2x2005_10.exe -h for help.\n");
 		 		return 1;
 		 }
 		 optch++;
	}
  	
	
	input = fopen(argv[2],"r");
	if (input == NULL)
	{
		fprintf(stderr,"\nFile is f00bar.\n");
		return 1;
	}

	int s = 0;	
	long file_size;
	fseek(input, 0, SEEK_END);	
	file_size = ftell(input);
	rewind(input);

	char* incipit = (char*)malloc(sizeof(char)*file_size);

	printf("###################################\n");
	printf("#                                 #\n");
	printf("# WELCOME TO P2X 2005 VERSION 1.0 #\n");
	printf("#                                 #\n");
	printf("#                                 #\n");
	printf("#     Press Enter to continue     #\n");
	printf("#                                 #\n");
	printf("###################################\n");
	getchar();
	for(s=0;s<25;s++) printf("\n");

	float cnt = 0;
	float progress = 0;
	float old_progress = 0;
	float nb_line = 52200.0f;
	
	errlog = fopen("p2x_errors.log","w");
	fprintf(errlog,"P2X 2005 ERROR LOG\n");
	fprintf(errlog,"------------------\n\n");

	printf("Starting conversion...\n");	
	while(1)
	{
		memset(incipit, 0, sizeof(char)*file_size);
		fgets(incipit, file_size, input);
		if (feof(input)) break;
		convert_incipit(incipit, file_size, mensural_remove, meas_rest_remove, verbose, xml);
		cnt++;
		old_progress = progress;
		progress = ((cnt/nb_line)*100.0f);
		if (((int)progress % 10 == 0) && ((int)progress != (int)old_progress))	printf("Processing... %d%c done.\n",(int) progress,'%');
	}
	fclose(input);
	fclose(errlog);

	printf("\nConversion done, press Enter to exit.\n");
	getchar();

	free(incipit);
	
	return 0;
}
